@@ -68,7 +68,6 @@
 #include "actioncollection.h"
 #include "cache.h"
 #include "collectionlist.h"
-#include "coverdialog.h"
 #include "coverinfo.h"
 #include "deletedialog.h"
 #include "directoryloader.h"
@@ -83,9 +82,9 @@
 #include "playlistsharedsettings.h"
 #include "tagtransactionmanager.h"
 #include "upcomingplaylist.h"
-#include "webimagefetcher.h"
 
 using namespace ActionCollection; // ""_act and others
+using std::as_const;
 
 /**
  * Used to give every track added in the program a unique identifier. See
@@ -124,7 +123,6 @@ Playlist::Playlist(
   , m_collection(collection)
   , m_playlistName(name)
   , m_refillDebounce(new QTimer(this))
-  , m_fetcher(new WebImageFetcher(this))
 {
     setup(extraCols);
 
@@ -308,9 +306,8 @@ void Playlist::playPrevious()
     PlaylistItem *previous = nullptr;
 
     if(random && !m_history.isEmpty()) {
-        PlaylistItemList::Iterator last = m_history.end() - 1;
-        previous = *last;
-        m_history.erase(last);
+        previous = m_history.last();
+        m_history.removeLast();
     }
     else {
         m_history.clear();
@@ -409,7 +406,7 @@ QStringList Playlist::files() const
 
 PlaylistItemList Playlist::items()
 {
-    return items(QTreeWidgetItemIterator::IteratorFlag(0));
+    return items(QTreeWidgetItemIterator::All);
 }
 
 PlaylistItemList Playlist::visibleItems()
@@ -455,6 +452,8 @@ void Playlist::setSearch(PlaylistSearch* s)
     for(int row = 0; row < topLevelItemCount(); ++row)
         topLevelItem(row)->setHidden(true);
     setItemsVisible(s->matchedItems(), true);
+
+    refillRandomList();
 }
 
 void Playlist::setSearchEnabled(bool enabled)
@@ -474,6 +473,8 @@ void Playlist::setSearchEnabled(bool enabled)
         for(PlaylistItem* item : playlistItems)
             item->setHidden(false);
     }
+
+    refillRandomList();
 }
 
 // Mostly seems to be for DynamicPlaylist
@@ -622,80 +623,6 @@ void Playlist::slotBeginPlayback()
     }
 }
 
-void Playlist::slotViewCover()
-{
-    const PlaylistItemList items = selectedItems();
-    for(const auto &item : items) {
-        const auto cover = item->file().coverInfo();
-
-        if(cover->hasCover()) {
-            cover->popup();
-            return; // If we select multiple items, only show one
-        }
-    }
-}
-
-void Playlist::slotRemoveCover()
-{
-    PlaylistItemList items = selectedItems();
-    if(items.isEmpty())
-        return;
-    int button = KMessageBox::warningContinueCancel(this,
-                                                    i18n("Are you sure you want to delete these covers?"),
-                                                    QString(),
-                                                    KGuiItem(i18n("&Delete Covers")));
-    if(button == KMessageBox::Continue)
-        refreshAlbums(items);
-}
-
-void Playlist::slotShowCoverManager()
-{
-    static CoverDialog *managerDialog = 0;
-
-    if(!managerDialog)
-        managerDialog = new CoverDialog(this);
-
-    managerDialog->show();
-}
-
-void Playlist::slotAddCover(bool retrieveLocal)
-{
-    PlaylistItemList items = selectedItems();
-
-    if(items.isEmpty())
-        return;
-
-    if(!retrieveLocal) {
-        m_fetcher->setFile((*items.begin())->file());
-        m_fetcher->searchCover();
-        return;
-    }
-
-    QUrl file = QFileDialog::getOpenFileUrl(
-        this, i18n("Select Cover Image File"),
-        QUrl::fromLocalFile(QDir::home().path()),
-        i18n("Images (*.png *.jpg)"), nullptr,
-        {}, QStringList() << QStringLiteral("file")
-        );
-    if(file.isEmpty())
-        return;
-
-    QString artist = items.front()->file().tag()->artist();
-    QString album = items.front()->file().tag()->album();
-
-    coverKey newId = CoverManager::addCover(file, artist, album);
-
-    if(newId != CoverManager::NoMatch)
-        refreshAlbums(items, newId);
-}
-
-// Called when image fetcher has added a new cover.
-void Playlist::slotCoverChanged(int coverId)
-{
-    qCDebug(JUK_LOG) << "Refreshing information for newly changed covers.";
-    refreshAlbums(selectedItems(), coverId);
-}
-
 void Playlist::slotGuessTagInfo(TagGuesser::Type type)
 {
     QApplication::setOverrideCursor(Qt::WaitCursor);
@@ -734,8 +661,6 @@ void Playlist::slotReload()
 
 void Playlist::refillRandomList()
 {
-    qCDebug(JUK_LOG) << "Refilling random items.";
-
     if(action("disableRandomPlay")->isChecked()) {
         m_randomSequence.clear();
         return;
@@ -743,6 +668,8 @@ void Playlist::refillRandomList()
 
     PlaylistItemList randomItems = visibleItems();
 
+    qCDebug(JUK_LOG) << "Refilling random items among" << randomItems.count() << "viable tracks";
+
     // See https://www.pcg-random.org/posts/cpp-seeding-surprises.html
     std::random_device rdev;
     uint64_t rseed = (uint64_t(rdev()) << 32) | rdev();
@@ -905,12 +832,6 @@ void Playlist::beginPlayingItem(PlaylistItem *itemToPlay)
 
 void Playlist::dragEnterEvent(QDragEnterEvent *e)
 {
-    if(CoverDrag::isCover(e->mimeData())) {
-        setDropIndicatorShown(false);
-        e->accept();
-        return;
-    }
-
     if(e->mimeData()->hasUrls() && !e->mimeData()->urls().isEmpty()) {
         setDropIndicatorShown(true);
         e->acceptProposedAction();
@@ -1026,32 +947,9 @@ bool Playlist::dropMimeData(QTreeWidgetItem *parent, int index, const QMimeData
 
 void Playlist::dropEvent(QDropEvent *e)
 {
-    QPoint vp = e->pos();
+    QPoint vp = e->position().toPoint();
     PlaylistItem *item = static_cast<PlaylistItem *>(itemAt(vp));
 
-    // First see if we're dropping a cover, if so we can get it out of the
-    // way early.
-    if(item && CoverDrag::isCover(e->mimeData())) {
-        coverKey id = CoverDrag::idFromData(e->mimeData());
-
-        // If the item we dropped on is selected, apply cover to all selected
-        // items, otherwise just apply to the dropped item.
-
-        if(item->isSelected()) {
-            const PlaylistItemList selItems = selectedItems();
-            for(auto &playlistItem : selItems) {
-                playlistItem->file().coverInfo()->setCoverId(id);
-                playlistItem->refresh();
-            }
-        }
-        else {
-            item->file().coverInfo()->setCoverId(id);
-            item->refresh();
-        }
-
-        return;
-    }
-
     // When dropping on the toUpper half of an item, insert before this item.
     // This is what the user expects, and also allows the insertion at
     // top of the list
@@ -1125,7 +1023,7 @@ void Playlist::read(QDataStream &s)
 
     m_blockDataChanged = true;
 
-    for(const auto &file : qAsConst(files)) {
+    for(const auto &file : as_const(files)) {
         if(file.isEmpty())
             throw BICStreamException();
 
@@ -1239,7 +1137,7 @@ void Playlist::addFiles(const QStringList &files, PlaylistItem *after)
 
     // Build handlers for all the still-active loaders on the heap and then
     // return to the event loop.
-    for(const auto &future : qAsConst(pendingFutures)) {
+    for(const auto &future : as_const(pendingFutures)) {
         auto loadWatcher = new QFutureWatcher<void>(this);
         loadWatcher->setFuture(future);
 
@@ -1253,27 +1151,6 @@ void Playlist::addFiles(const QStringList &files, PlaylistItem *after)
     }
 }
 
-void Playlist::refreshAlbums(const PlaylistItemList &items, coverKey id)
-{
-    QSet<QPair<QString, QString>> albums;
-    bool setAlbumCovers = items.count() == 1;
-
-    for(const auto &item : items) {
-        const QString artist = item->file().tag()->artist();
-        const QString album = item->file().tag()->album();
-
-        albums.insert(qMakePair(artist, album));
-
-        item->file().coverInfo()->setCoverId(id);
-        if(setAlbumCovers)
-            item->file().coverInfo()->applyCoverToWholeAlbum(true);
-    }
-
-    for(const auto &albumPair : qAsConst(albums)) {
-        refreshAlbum(albumPair.first, albumPair.second);
-    }
-}
-
 void Playlist::updatePlaying() const
 {
     const auto playingItems = PlaylistItem::playingItems();
@@ -1403,7 +1280,6 @@ void Playlist::slotInitialize(int numColumnsToReserve)
         i18n("Track Name"),
         i18n("Artist"),
         i18n("Album"),
-        i18n("Cover"),
         i18nc("cd track number", "Track"),
         i18n("Genre"),
         i18n("Year"),
@@ -1531,22 +1407,20 @@ void Playlist::slotPopulateBackMenu() const
     if(!playingItem())
         return;
 
-    QMenu *menu = action<KToolBarPopupAction>("back")->menu();
+    QMenu *menu = action<KToolBarPopupAction>("back")->popupMenu();
     menu->clear();
     m_backMenuItems.clear();
     m_backMenuItems.reserve(10);
 
     int count = 0;
-    PlaylistItemList::ConstIterator it = m_history.constEnd();
 
-    QAction *action;
+    for(auto it = m_history.crbegin(); it != m_history.crend(); it++) {
+        QAction *action = new QAction((*it)->file().tag()->title(), menu);
+        action->setData(count);
 
-    while(it != m_history.constBegin() && count < 10) {
-        ++count;
-        --it;
-        action = new QAction((*it)->file().tag()->title(), menu);
-        action->setData(count - 1);
         menu->addAction(action);
+
+        ++count;
         m_backMenuItems << *it;
     }
 }
@@ -1579,14 +1453,6 @@ void Playlist::setup(int numColumnsToReserve)
     connect(header(), &QHeaderView::sectionMoved,
             this,     &Playlist::slotColumnOrderChanged);
 
-    connect(m_fetcher, &WebImageFetcher::signalCoverChanged,
-            this,      &Playlist::slotCoverChanged);
-
-    // Prevent list of selected items from changing while internet search is in
-    // progress.
-    connect(this,      &Playlist::itemSelectionChanged,
-            m_fetcher, &WebImageFetcher::abortSearch);
-
     connect(this, &QTreeWidget::itemDoubleClicked,
             this, &Playlist::slotPlayCurrent);
 
@@ -1666,7 +1532,8 @@ void Playlist::setPlaying(PlaylistItem *item, bool addToHistory)
         m_history.append(wasPlayingItem->collectionItem());
 
         const bool enableBack = !m_history.isEmpty();
-        action<KToolBarPopupAction>("back")->menu()->setEnabled(enableBack);
+        const auto menu = action<KToolBarPopupAction>("back")->popupMenu();
+        menu->setEnabled(enableBack);
     }
 
     if(wasPlayingItem) {
@@ -1734,8 +1601,6 @@ void Playlist::createPlaylistRMBMenu()
     m_rmbMenu->addAction(action("guessTag"));
     m_rmbMenu->addAction(action("renameFile"));
 
-    m_rmbMenu->addAction(action("coverManager"));
-
     m_rmbMenu->addSeparator();
 
     m_rmbMenu->addAction("folder-new"_icon,
@@ -1788,7 +1653,7 @@ void Playlist::calculateColumnWeights()
     if(m_columnWeights.isEmpty())
         m_columnWeights.fill(-1, columnCount());
 
-    for(const auto column : qAsConst(m_weightDirty)) {
+    for(const auto column : as_const(m_weightDirty)) {
         m_columnWeights[column] = int(std::sqrt(averageWidth[column]) + 0.5);
     }
 
@@ -1819,7 +1684,7 @@ QFuture<void> Playlist::addFilesFromDirectory(const QString &dirPath)
         }
     );
 
-    auto future = QtConcurrent::run(loader, &DirectoryLoader::startLoading);
+    auto future = QtConcurrent::run(&DirectoryLoader::startLoading, loader);
     auto loadWatcher = new QFutureWatcher<void>(this);
     connect(loadWatcher, &QFutureWatcher<void>::finished, this, [=]() {
             loader->deleteLater();
@@ -1918,7 +1783,7 @@ void Playlist::slotUpdateColumnWidths()
 
     // No item content to auto-fit around, use the headers for now
     if(count() == 0) {
-        for(int column : qAsConst(visibleColumns)) {
+        for(int column : as_const(visibleColumns)) {
             setColumnWidth(column, textWidth(header()->fontMetrics(),headerItem()->text(column)) + 10);
         }
 
@@ -1940,7 +1805,7 @@ void Playlist::slotUpdateColumnWidths()
     QVector<int> minimumFixedWidth(columnCount(), 0);
     int minimumFixedWidthTotal = 0;
 
-    for(int column : qAsConst(visibleColumns)) {
+    for(int column : as_const(visibleColumns)) {
         minimumWidth[column] = textWidth(header()->fontMetrics(), headerItem()->text(column)) + 10;
         minimumWidthTotal += minimumWidth[column];
 
@@ -1967,7 +1832,7 @@ void Playlist::slotUpdateColumnWidths()
     // useful weight to use as a divisor for each column's weight.
 
     double totalWeight = 0;
-    for(int column : qAsConst(visibleColumns)) {
+    for(int column : as_const(visibleColumns)) {
         totalWeight += m_columnWeights[column];
     }
 
@@ -1980,7 +1845,7 @@ void Playlist::slotUpdateColumnWidths()
     // width if we didn't have to handle the cases of minimum and maximum widths.
 
     QVector<int> weightedWidth(columnCount(), 0);
-    for(int column : qAsConst(visibleColumns)) {
+    for(int column : as_const(visibleColumns)) {
         weightedWidth[column] = int(double(m_columnWeights[column]) / totalWeight * viewport()->width() + 0.5);
     }
 
@@ -2012,7 +1877,7 @@ void Playlist::slotUpdateColumnWidths()
 
     // Fill in the values discussed above.
 
-    for(int column : qAsConst(visibleColumns)) {
+    for(int column : as_const(visibleColumns)) {
         if(weightedWidth[column] < minimumWidth[column]) {
             readjust = true;
             extraWidth[column] = 0;
@@ -2038,7 +1903,7 @@ void Playlist::slotUpdateColumnWidths()
     // than the minimum widths, just use those, otherwise use the "readjusted
     // weighted width".
 
-    for(int column : qAsConst(visibleColumns)) {
+    for(int column : as_const(visibleColumns)) {
         int width;
         if(readjust) {
             int adjustedExtraWidth = int(double(extraWidth[column]) * adjustmentRatio + 0.5);
@@ -2091,7 +1956,18 @@ void Playlist::slotShowRMBMenu(const QPoint &point)
         (adjColumn == PlaylistItem::YearColumn);
 
     if(showEdit) {
-        m_rmbEdit->setText(i18n("Edit '%1'", item->text(column)));
+        QString labelText(item->text(column));
+
+        // Don't show an ugly “Edit ''”
+        if(labelText.isEmpty()) {
+            labelText = headerItem()->text(column);
+        }
+        if(labelText.length() > 31) {
+            labelText.truncate(31);
+            labelText.append(QStringLiteral("..."));
+        }
+
+        m_rmbEdit->setText(i18n("Edit '%1'", labelText));
 
         m_rmbEdit->disconnect(this);
         connect(m_rmbEdit, &QAction::triggered, this, [this, item, column]() {
@@ -2107,13 +1983,6 @@ void Playlist::slotShowRMBMenu(const QPoint &point)
 
     m_rmbEdit->setEnabled(file.fileInfo().isWritable() || selectedItems().count() > 1);
 
-    // View cover is based on if there is a cover to see.  We should only have
-    // the remove cover option if the cover is in our database (and not directly
-    // embedded in the file, for instance).
-
-    action("viewCover")->setEnabled(file.coverInfo()->hasCover());
-    action("removeCover")->setEnabled(file.coverInfo()->coverId() != CoverManager::NoMatch);
-
     m_rmbMenu->popup(mapToGlobal(point));
 }
 
